@@ -510,14 +510,17 @@ class BaseTransformer(torch.nn.Module):
         assert len(attention_mask.shape) == 2 or \
                len(attention_mask.shape) == 4 and attention_mask.shape[1] == 1
 
+        # initial output_cross_layer might be generated by word/position_embedding_forward
+        output_cross_layer = {}
+
         # embedding part
         if 'word_embedding_forward' in self.hooks:
-            hidden_states = self.hooks['word_embedding_forward'](input_ids, **kw_args)
+            hidden_states = self.hooks['word_embedding_forward'](input_ids, output_cross_layer=output_cross_layer, **kw_args)
         else:  # default
             hidden_states = self.word_embeddings(input_ids)
 
         if 'position_embedding_forward' in self.hooks:
-            position_embeddings = self.hooks['position_embedding_forward'](position_ids, **kw_args)
+            position_embeddings = self.hooks['position_embedding_forward'](position_ids, output_cross_layer=output_cross_layer, **kw_args)
         else:
             assert len(position_ids.shape) <= 2
             assert position_ids.shape[-1] == query_length
@@ -526,12 +529,6 @@ class BaseTransformer(torch.nn.Module):
             hidden_states = hidden_states + position_embeddings
         hidden_states = self.embedding_dropout(hidden_states)
 
-        # initial output_cross_layer
-        if 'cross_layer_embedding_forward' in self.hooks:
-            output_cross_layer = self.hooks['cross_layer_embedding_forward'](hidden_states, **kw_args)
-        else:
-            output_cross_layer = {}
-
         output_per_layers = []
         if self.checkpoint_activations:
             # define custom_forward for checkpointing
