@@ -30,10 +30,7 @@ def get_checkpoint_name(checkpoints_path, iteration, release=False, zero=False):
 
 
 def get_checkpoint_tracker_filename(checkpoints_path, old_checkpoint=False):
-    if old_checkpoint:
-        return os.path.join(checkpoints_path, 'latest_checkpointed_iteration.txt')
-    else:
-        return os.path.join(checkpoints_path, 'latest')
+    return os.path.join(checkpoints_path, 'latest')
 
 
 def save_checkpoint(iteration, model, optimizer,
@@ -90,7 +87,7 @@ def save_ds_checkpoint_no_optim(model, save_dir, tag=None, client_state={}, save
 
 def get_checkpoint_iteration(args):
     # Read the tracker file and set the iteration.
-    tracker_filename = get_checkpoint_tracker_filename(args.load, old_checkpoint=args.old_checkpoint)
+    tracker_filename = get_checkpoint_tracker_filename(args.load)
     if not os.path.isfile(tracker_filename):
         print_rank_0('could not find the metadata file {} '.format(
             tracker_filename))
@@ -134,12 +131,6 @@ def load_checkpoint(model, args):
     else: # inference without deepspeed
         module = model
 
-    # sd['module']['transformer.word_embeddings.weight'] = sd['module']['word_embeddings.weight']
-    # del sd['module']['word_embeddings.weight']
-    # sd['module']['mixins.block_position_embedding.block_position_embeddings.weight'] = sd['module'][
-    #     'transformer.block_position_embeddings.weight']
-    # del sd['module']['transformer.block_position_embeddings.weight']
-
     # only load module, other hyperparameters are just for recording.
     missing_keys, unexpected_keys = module.load_state_dict(sd['module'], strict=False)
     if len(unexpected_keys) > 0:
@@ -152,7 +143,7 @@ def load_checkpoint(model, args):
             assert all(name.find('mixins')>=0 for name in missing_keys)
             assert args.mode == 'finetune'
             module.reinit() # initialize mixins
-    if args.mode != 'inference':
+    if args.mode != 'inference' and args.deepspeed and args.fp16:
         model.optimizer.refresh_fp32_params() # restore fp32 weights from module
 
     # Iterations.
